﻿Module MainModule

    Private _rng As Random

    Sub Main()
        _rng = New Random()
        Dim roster() As Employee = CreateEmployees()

        ' Sort employees by ID
        Console.WriteLine("Employees sorted by ID:")
        Array.Sort(roster, AddressOf IdSorter)
        PrintEmployees(roster)

        ' Sort employees by Name
        Console.WriteLine("Employees sorted by name:")
        Array.Sort(roster, AddressOf NameSorter)
        PrintEmployees(roster)
    End Sub

#Region "Non-demo Code"
    ' Generates a random number of employees with random names and random ID numbers.
    Private Function CreateEmployees() As Employee()
        Dim employees(_rng.Next(5, 8)) As Employee
        Dim firstNames() As String = {"James", "Sally", "Bob", "Alice", "Karen", "Sam"}
        Dim lastNames() As String = {"Allen", "Smith", "Doe", "Williamson", "Travis", "Tyler"}

        For i As Integer = 0 To employees.Length - 1
            Dim emp As New Employee()
            emp.Id = _rng.Next(100000, 999999) ' Not guaranteed to be unique, but likely!
            emp.Name = String.Format("{0} {1}", firstNames(_rng.Next(0, firstNames.Length)), _
                                                lastNames(_rng.Next(0, lastNames.Length)))
            employees(i) = emp
        Next

        Return employees
    End Function

    Private Sub PrintEmployees(ByVal roster() As Employee)
        For Each emp As Employee In roster
            Console.WriteLine(emp.ToString())
        Next
    End Sub
#End Region

    ' Compares Employee objects based on their employee ID
    Private Function IdSorter(ByVal left As Employee, ByVal right As Employee) As Integer
        If left.Id < right.Id Then
            Return -1
        ElseIf left.Id > right.Id Then
            Return 1
        Else
            Return 0
        End If
    End Function

    'Compares employee objects based on the employee name; follows the convention that last name
    ' is sorted before first name, and the case of names is ignored.
    Private Function NameSorter(ByVal left As Employee, ByVal right As Employee) As Integer
        Dim leftName() As String = left.Name.Split(" "c)
        Dim rightname() As String = right.Name.Split(" "c)

        ' Compare last names
        Dim compareResult As Integer = String.Compare(leftName(1), rightname(1), True)

        ' If last names are the same, compare first names
        If compareResult = 0 Then
            compareResult = String.Compare(leftName(0), rightname(0), True)
        End If

        Return compareResult
    End Function

End Module
