﻿Public Class TemperatureSensor
    Implements ISensor

    Private _observers As List(Of IObserver)
    Private _rng As Random
    Private _temperatureReading As Double
    Private _updateTimer As Timer

    Public Sub New()
        _observers = New List(Of IObserver)()
        _rng = New Random()
        _temperatureReading = 70.0
        _updateTimer = New Timer()
        _updateTimer.Interval = 1000

        AddHandler _updateTimer.Tick, AddressOf UpdateTimerTicked
        _updateTimer.Start()
    End Sub

    Public ReadOnly Property Data() As Double Implements ISensor.Data
        Get
            Return _temperatureReading
        End Get
    End Property

    Public ReadOnly Property Name() As String Implements ISensor.Name
        Get
            Return "Temperature"
        End Get
    End Property

    Public Sub AddListener(ByVal listener As IObserver) Implements IObservable.AddListener
        _observers.Add(listener)
    End Sub

    Public Sub Notify() Implements IObservable.Notify
        For Each o In _observers
            o.Update()
        Next
    End Sub

    Public Sub RemoveListener(ByVal listener As IObserver) Implements IObservable.RemoveListener
        _observers.Remove(listener)
    End Sub

    Private Sub UpdateTimerTicked(ByVal sender As Object, ByVal e As EventArgs)
        ' Check the temperature sensor hardware; here we just make a random number
        _temperatureReading = _rng.NextDouble * 20 + 70

        ' We changed the temperature; notify all observers
        Notify()
    End Sub
End Class
