﻿Public Class TemperatureSensor
    Implements ISensor

    Private _rng As Random
    Private _temperatureReading As Double
    Private _updateCallbacks As List(Of ISensor.UpdateCallback)
    Private _updateTimer As Timer

    Public Sub New()
        _rng = New Random()
        _temperatureReading = 70.0
        _updateCallbacks = New List(Of ISensor.UpdateCallback)
        _updateTimer = New Timer()
        _updateTimer.Interval = 1000

        AddHandler _updateTimer.Tick, AddressOf UpdateTimerTicked
        _updateTimer.Start()
    End Sub

    Public ReadOnly Property Data() As Double Implements ISensor.Data
        Get
            Return _temperatureReading
        End Get
    End Property

    Public ReadOnly Property Name() As String Implements ISensor.Name
        Get
            Return "Temperature"
        End Get
    End Property

    Public Sub AddCallback(ByVal callback As ISensor.UpdateCallback) Implements ISensor.AddCallback
        _updateCallbacks.Add(callback)
    End Sub

    Public Sub RemoveCallback(ByVal callback As ISensor.UpdateCallback) Implements ISensor.RemoveCallback
        _updateCallbacks.Remove(callback)
    End Sub

    Private Sub UpdateTimerTicked(ByVal sender As Object, ByVal e As EventArgs)
        ' Check the temperature sensor hardware; here we just make a random number
        _temperatureReading = _rng.NextDouble * 20 + 70

        ' We changed the temperature; notify observer
        OnTemperatureUpdated()
    End Sub

    Private Sub OnTemperatureUpdated()
        For Each callback As ISensor.UpdateCallback In _updateCallbacks
            callback.Invoke(Data)
        Next
    End Sub

End Class
