﻿Public Class TemperatureSensor
    Implements ISensor

    Private _rng As Random
    Private _temperatureReading As Double
    Private _updateTimer As Timer

    Public Sub New()
        _rng = New Random()
        _temperatureReading = 70.0
        _updateTimer = New Timer()
        _updateTimer.Interval = 1000

        AddHandler _updateTimer.Tick, AddressOf UpdateTimerTicked
        _updateTimer.Start()
    End Sub

    Public ReadOnly Property Data() As Double Implements ISensor.Data
        Get
            Return _temperatureReading
        End Get
    End Property

    Public ReadOnly Property Name() As String Implements ISensor.Name
        Get
            Return "Temperature"
        End Get
    End Property

    Public ReadOnly Property Temperature() As Double
        Get
            Return _temperatureReading
        End Get
    End Property

    Private Sub UpdateTimerTicked(ByVal sender As Object, ByVal e As EventArgs)
        ' Check the temperature sensor hardware; here we just make a random number
        _temperatureReading = _rng.NextDouble * 20 + 70

        ' We changed the temperature; notify all observers
        OnUpdate(New UpdateEventArgs(Temperature))
    End Sub

    Private Sub OnUpdate(ByVal e As UpdateEventArgs)
        RaiseEvent Update(Me, e)
    End Sub

    Public Event Update As EventHandler(Of UpdateEventArgs) Implements ISensor.Update

End Class
