﻿Public Class TemperatureSensor
    Implements ISensor

    Private _rng As Random
    Private _temperatureReading As Double
    Private _updateCallback As ISensor.UpdateCallback
    Private _updateTimer As Timer

    Public Sub New()
        _rng = New Random()
        _temperatureReading = 70.0
        _updateTimer = New Timer()
        _updateTimer.Interval = 1000

        AddHandler _updateTimer.Tick, AddressOf UpdateTimerTicked
        _updateTimer.Start()
    End Sub

    Public ReadOnly Property Data() As Double Implements ISensor.Data
        Get
            Return _temperatureReading
        End Get
    End Property

    Public ReadOnly Property Name() As String Implements ISensor.Name
        Get
            Return "Temperature"
        End Get
    End Property

    Public ReadOnly Property Temperature() As Double
        Get
            Return _temperatureReading
        End Get
    End Property

    Public Sub AddCallback(ByVal callback As ISensor.UpdateCallback) Implements ISensor.AddCallback
        _updateCallback = [Delegate].Combine(_updateCallback, callback)
    End Sub

    Public Sub RemoveCallback(ByVal callback As ISensor.UpdateCallback) Implements ISensor.RemoveCallback
        _updateCallback = [Delegate].Remove(_updateCallback, callback)
    End Sub

    Private Sub UpdateTimerTicked(ByVal sender As Object, ByVal e As EventArgs)
        ' Check the temperature sensor hardware; here we just make a random number
        _temperatureReading = _rng.NextDouble * 20 + 70

        ' We changed the temperature; notify all observers
        OnTemperatureUpdated()
    End Sub

    Private Sub OnTemperatureUpdated()
        If _updateCallback IsNot Nothing Then
            _updateCallback.Invoke(Temperature)
        End If
    End Sub

End Class
